@@ -1083,12 +1083,14 @@ int PnmReadHeader(TEOFILE *teofp){
       TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
       return  0;
     }
-    if(graylevel > 255){
-      TeoSetErrorMessage(TEO_ER_READ_WRONG_PSIZE,teofp->filename);
-      return 0;
+    if(graylevel < 256){
+      teofp->bit = 8;
+    } else if(graylevel < 65536){
+      teofp->bit = 16;
+    } else {
+      teofp->bit = 32;
     }
     teofp->type = TEO_UNSIGNED;
-    teofp->bit = 8;
     teofp->plane = 1;
     teofp->frame = 1;
   } else if(memcmp(magic,"P6",2) == 0){
@@ -1107,12 +1109,14 @@ int PnmReadHeader(TEOFILE *teofp){
       TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
       return  0;
     }
-    if(graylevel > 255){
-      TeoSetErrorMessage(TEO_ER_READ_WRONG_PSIZE,teofp->filename);
-      return 0;
+    if(graylevel < 256){
+      teofp->bit = 8;
+    } else if(graylevel < 65536){
+      teofp->bit = 16;
+    } else {
+      teofp->bit = 32;
     }
     teofp->type = TEO_UNSIGNED;
-    teofp->bit = 8;
     teofp->plane = 3;
     teofp->frame = 1;
   } else {
@@ -1147,10 +1151,10 @@ int PnmWriteHeader(TEOFILE *teofp){
   if(teofp->bit == 1 &&
      teofp->plane == 1)
     strcpy(magic,"P4");
-  else if(teofp->bit == 8 &&
+  else if(teofp->type == TEO_UNSIGNED &&
 	    teofp->plane == 1)
     strcpy(magic,"P5");
-  else if(teofp->bit == 8 &&
+  else if(teofp->type == TEO_UNSIGNED &&
 	  teofp->plane == 3)
     strcpy(magic,"P6");
   else {
@@ -1163,9 +1167,18 @@ int PnmWriteHeader(TEOFILE *teofp){
 
   /* image size */
   fprintf(teofp->fp,"%d %d\012",teofp->width,teofp->height);
-  if(strcmp(magic,"P4"))
-    fprintf(teofp->fp,"255\012");
-
+  if(strcmp(magic,"P4")){
+    if(teofp->bit == 8) 
+      fprintf(teofp->fp,"255\012");
+    else if(teofp->bit == 16)
+      fprintf(teofp->fp,"65535\012");
+    else if(teofp->bit == 32)
+      fprintf(teofp->fp,"4294967295\01b2");
+    else {
+      TeoSetErrorMessage(TEO_ER_WRITE_WRONG_PTYPE,teofp->filename);
+      return 0;
+    }
+  }
   teofp->fsize = (teofp->bit == 1)? 
     (((int)((teofp->width-1)/8) + 1) * teofp->height * teofp->plane) : 
     (teofp->width * teofp->height * (teofp->bit/8) * teofp->plane);
